

package com.aliyun.odps.spark.examples.sparksql;

import com.aliyun.odps.Odps;
import com.aliyun.odps.cupid.CupidSession;
import com.tzld.piaoquan.data.base.UserFeature;
import com.tzld.piaoquan.data.dataloader.FeatureConstructor;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;
import org.apache.spark.api.java.function.Function;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.RowFactory;
import org.apache.spark.sql.types.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.spark.sql.types.StructField;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.StringRedisSerializer;

public class JavaSparkSQL {

    private static final String userKeyFormat = "user:%s";

    public static RedisTemplate<String, String> buildRedisTemplate() {
        RedisStandaloneConfiguration rsc = new RedisStandaloneConfiguration();
        rsc.setPort(6379);
        rsc.setPassword("Wqsd@2019");
        rsc.setHostName("r-bp1ps6my7lzg8rdhwx682.redis.rds.aliyuncs.com");
        RedisTemplate<String, String> template = new RedisTemplate<>();
        JedisConnectionFactory fac = new JedisConnectionFactory(rsc);
        fac.afterPropertiesSet();
        template.setDefaultSerializer(new StringRedisSerializer());
        template.setConnectionFactory(fac);
        template.afterPropertiesSet();
        return template;
    }

    public static void loadFeatureToRedis(RedisTemplate<String, String> redisTemplate, Row line) {
        Map<String, String> userFeaRedisFormat = new HashMap<String, String>();
        UserFeature userFeature = FeatureConstructor.constructUserFeature(line);
        String key = String.format(userKeyFormat, userFeature.getKey());
        String value = userFeature.getValue();
        userFeaRedisFormat.put(key, value);
        redisTemplate.opsForValue().multiSet(userFeaRedisFormat);
    }



    public static void main(String[] args) throws Exception {
        SparkSession spark = SparkSession
                .builder()
                .appName("SparkSQL-on-MaxCompute")
                .config("spark.sql.defaultCatalog","odps")
                .config("spark.sql.catalog.odps", "org.apache.spark.sql.execution.datasources.v2.odps.OdpsTableCatalog")
                .config("spark.sql.sources.partitionOverwriteMode", "dynamic")
                .config("spark.sql.extensions", "org.apache.spark.sql.execution.datasources.v2.odps.extension.OdpsExtensions")
                .config("spark.sql.catalogImplementation","hive")
                .getOrCreate();
        JavaSparkContext sparkContext = new JavaSparkContext(spark.sparkContext());



        // 读 分区表
        Dataset<Row> rptdf = spark.sql("select * from loghubods.alg_recsys_user_info where dt='20231210' limit 1000");
        System.out.println("rptdf count: " + rptdf.count());
        rptdf.printSchema();


        rptdf.toJavaRDD().foreachPartition(
                rowIterator -> {
                    RedisTemplate<String, String> redisTemplate = buildRedisTemplate();
                    rowIterator.forEachRemaining(line -> loadFeatureToRedis(redisTemplate, line));
                }
        );


    }
}